public class ProducerConsumer {
	static Object monitor = new Object();
	static Object monitor2 = new Object();
	static int val = 0;
	static boolean ready = false;
	
	public static void put(int n) throws InterruptedException {
		synchronized (monitor) {
			while(ready) {
				//System.out.println("IN " + n);
				monitor.wait();
				//System.out.println("OUT " + n);
			}
			val = n;
			ready = true;
			monitor.notifyAll();
			//System.out.println("PUT RETURN");
		}
	}
	
	public static int get() throws InterruptedException {
		synchronized (monitor) {
			while (!ready) {
				//System.out.println("GET IN");
				monitor.wait();
				//System.out.println("GET OUT");
			}
			ready = false;
			monitor.notify();
			//System.out.println("GET RETURN");
			return val;
		}
	}
	
	static class PrimeProducer extends Thread {
		boolean isPrime(int p) {
			for (int d = 2; d <= Math.round(Math.sqrt(p)); d++) {
				if (p % d == 0) {
					return false;
				}
			}
			return true;
		}
		
		public void run() {
			try {
				for (int i = 2; i < Integer.MAX_VALUE; i++) {
					if (isPrime(i)) {
						put(i);
					}
				}
			}
			catch (InterruptedException e) {}
		}
	}
	
	static class PerfectProducer extends Thread {
		boolean isPerfect(int p) {
			int sum = 1;
			for (int d = 2; d <= Math.round(Math.sqrt(p)); d++) {
				if (p % d == 0) {
					sum += d;
					int div = p / d;
					if (div != d) {
						sum += div;;
					}
				}
			}
			return sum == p;
		}
		
		public void run() {
			try {
				for (int i = 2; i < Integer.MAX_VALUE; i++) {
					if (isPerfect(i)) {
						put(i);
					}
				}
			}
			catch (InterruptedException e) {}
		}
	}
	
	static class Consumer extends Thread {
		public void run() {
			try {
				for (int k = 0; k < 100000; k++) {
					int v = get();
					System.out.printf("%d ", v);
					if (k % 20 == 0) {
						System.out.println();
					}
				}
			}
			catch (InterruptedException e) {}
		}
	}
	
	public static void main(String[] args) {
		Thread primes = new PrimeProducer();
		Thread perfects = new PerfectProducer();
		Thread consumer = new Consumer();
		
		primes.start();
		perfects.start();
		consumer.start();
		
		try {
			consumer.join();
		}
		catch (InterruptedException e) {}
		primes.stop();
		perfects.stop();
	}
}
